@@ -4,18 +4,22 @@
 "*passwdTimeout:		30",
 "*nice:			10",
 "*lock:			False",
+"*lockVTs:		True",
 "*verbose:		False",
 "*timestamp:		False",
 "*fade:			True",
 "*unfade:		False",
 "*fadeSeconds:		3",
 "*fadeTicks:		20",
+"*splash:		True",
+"*splashDuration:	5",
 "*captureStderr: 	True",
-"*captureStdout: 	True",
 "*overlayTextForeground:	#FFFF00",
 "*overlayTextBackground:	#000000",
 "*overlayStderr:		True",
 "*font:			*-medium-r-*-140-*-m-*",
+"*helpURL: http://www.jwz.org/xscreensaver/man.html",
+"*loadURL: netscape -remote 'openURL(%s)' || netscape '%s' &",
 "*installColormap:	True",
 "*programs:	qix -root -solid -delay 0 -segments 100			\\n\
 		attraction -root -mode balls				\\n\
@@ -116,13 +120,49 @@
 		pipes -root					  	\\n\
 		sproingies -root				  	\\n\
 		rubik -root					  	\\n\
-		atlantis -root					  	\\n",
+		atlantis -root					  	\\n\
+		lament -root					  	\\n\
+		bubble3d -root					  	\\n\
+		glplanet -root					  	\\n",
 " ",
+"*Dialog.headingFont:		*-times-bold-r-*-*-*-180-*-*-*-iso8859-1",
+"*Dialog.bodyFont:		*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
+"*Dialog.labelFont:		*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
+"*Dialog.buttonFont:		*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
+"*Dialog.foreground:		#000000",
+"*Dialog.background:		#BFBFBF",
+"*Dialog.Button.foreground:	#000000",
+"*Dialog.Button.background:	#D0D0D0",
+"*Dialog.text.foreground:	#000000",
+"*Dialog.text.background:	#FFFFFF",
+"*Dialog.logo.foreground:	#FF0000",
+"*Dialog.logo.background:	#FFFFFF",
+"*Dialog.topShadowColor:		#E7E7E7",
+"*Dialog.bottomShadowColor:	#737373",
+"*Dialog.logo.width:		200",
+"*Dialog.logo.height:		200",
+"*Dialog.internalBorderWidth:	30",
+"*Dialog.borderWidth:		1",
+"*Dialog.shadowThickness:	4",
+"*passwd.heading.label:		XScreenSaver %s",
+"*passwd.body.label:		This display is locked.",
+"*passwd.user.label:		User:",
+"*passwd.passwd.label:		Password:",
+"*passwd.passwdFont:		*-courier-medium-r-*-*-*-140-*-*-*-iso8859-1",
+"*passwd.thermometer.width:	8",
+"*splash.heading.label:		XScreenSaver %s",
+"*splash.body.label:		Copyright  1991-1998 by",
+"*splash.body2.label:		Jamie Zawinski <jwz@jwz.org>",
+"*splash.demo.label:		Demo",
+"*splash.prefs.label:		Prefs",
+"*splash.help.label:		Help",
 "*fontList:                       *-helvetica-medium-r-*-*-*-120-*-*-*-iso8859-1",
 "*demoDialog*label1.fontList:     *-helvetica-medium-r-*-*-*-140-*-*-*-iso8859-1",
-"*passwdDialog*fontList:          *-helvetica-medium-r-*-*-*-140-*-*-*-iso8859-1",
 "*XmTextField.fontList:             *-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
-"*passwdDialog.passwdText.fontList: *-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
+"*splashDialog*fontList:		   *-helvetica-bold-r-*-*-*-120-*-*-*-iso8859-1",
+"*splashDialog*splashLabel1.fontList:   *-times-bold-r-*-*-*-180-*-*-*-iso8859-1",
+"*splashDialog*splashLabel2.fontList:   *-times-bold-r-*-*-*-140-*-*-*-iso8859-1",
+"*splashDialog*splashLabel3.fontList:   *-times-bold-r-*-*-*-140-*-*-*-iso8859-1",
 "XScreenSaver*XmList.fontList:      *-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
 "*XmDialogShell*foreground:		#000000",
 "*XmDialogShell*background:		#E5E5E5",
@@ -138,8 +178,8 @@
 "*demoDialog.maxWidth:		600",
 "*label1.labelString:		XScreenSaver %s",
 "*label1.label:			XScreenSaver %s",
-"*label2.labelString: Copyright  1991-1998 by Jamie Zawinski <jwz@netscape.com>",
-"*label2.label:	     Copyright  1991-1998 by Jamie Zawinski <jwz@netscape.com>",
+"*label2.labelString: Copyright  1991-1998 by Jamie Zawinski <jwz@jwz.org>",
+"*label2.label:	     Copyright  1991-1998 by Jamie Zawinski <jwz@jwz.org>",
 "*demoList.visibleItemCount:	10",
 "*demoList.automaticSelection:	True",
 "*next.labelString:		Run Next",
@@ -162,21 +202,26 @@
 "*lockToggle.labelString:	Require Password",
 "*resourcesDone.labelString:	OK",
 "*resourcesCancel.labelString:	Cancel",
-"*passwdDialog.title:		Password",
-"*passwdLabel1.labelString:	XScreenSaver %s",
-"*passwdLabel2.labelString:	This display is locked.",
-"*passwdLabel3.labelString:	Please type %s's password to unlock it.",
-"*passwdDone.labelString:	OK",
-"*passwdCancel.labelString:	Cancel",
-"*passwdLabel1.alignment:	ALIGNMENT_BEGINNING",
-"*passwdLabel2.alignment:	ALIGNMENT_BEGINNING",
-"*passwdLabel3.alignment:	ALIGNMENT_BEGINNING",
-"*rogerLabel.width:		150",
+"*splashDialog.title:		XScreenSaver",
+"*splashLabel1.labelString:	XScreenSaver %s",
+"*splashLabel2.labelString:	Copyright  1991-1998 by",
+"*splashLabel3.labelString:	Jamie Zawinski <jwz@jwz.org>",
+"*splashDemo.labelString:	Demo",
+"*splashPrefs.labelString:	Prefs",
+"*splashDialog*Help.labelString:	Help",
+"*splashLabel1.alignment:	ALIGNMENT_CENTER",
+"*splashLabel2.alignment:	ALIGNMENT_CENTER",
+"*splashLabel3.alignment:	ALIGNMENT_CENTER",
+"*splashDialog.borderWidth:	1",
+"*splashForm.shadowThickness:	4",
 "XScreenSaver*dragInitiatorProtocolStyle: DRAG_NONE",
 "XScreenSaver*dragReceiverProtocolStyle:  DRAG_NONE",
 "*demo_dialog*font:		*-helvetica-bold-r-*-*-*-120-*-*-*-iso8859-1",
 "*resources_dialog*font:		*-helvetica-bold-r-*-*-*-120-*-*-*-iso8859-1",
-"*passwd_dialog*font:		*-helvetica-bold-r-*-*-*-120-*-*-*-iso8859-1",
+"*splash_dialog*font:		*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
+"*splash_dialog*splashLabel1.font:   *-times-bold-r-*-*-*-180-*-*-*-iso8859-1",
+"*splash_dialog*splashLabel2.font:   *-times-bold-r-*-*-*-140-*-*-*-iso8859-1",
+"*splash_dialog*splashLabel3.font:   *-times-bold-r-*-*-*-140-*-*-*-iso8859-1",
 "*demo_dialog*label1.font:	*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
 "*resources_dialog*label1.font:	*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
 "XScreenSaver.demo_dialog*List.font: \
@@ -196,12 +241,12 @@
 "XScreenSaver.resources_dialog*Text*background:		#FFFFFF",
 "*resources_dialog*Dialog.value.translations: #override\\n\
 	<Key>Return: beginning-of-line()\\n",
-"XScreenSaver.passwd_dialog*foreground:			#000000",
-"XScreenSaver.passwd_dialog*background:			#E5E5E5",
-"XScreenSaver.passwd_dialog*Command.background:		#D9D9D9",
-"XScreenSaver.passwd_dialog*Text*background:		#FFFFFF",
-"XScreenSaver.passwd_dialog*roger.foreground:		#CD0000",
-"XScreenSaver.passwd_dialog*roger.background:		#FFFFFF",
+"XScreenSaver.splash_dialog*foreground:			#000000",
+"XScreenSaver.splash_dialog*background:			#E5E5E5",
+"XScreenSaver.splash_dialog*Command.background:		#D9D9D9",
+"XScreenSaver.splash_dialog*Text*background:		#FFFFFF",
+"XScreenSaver.splash_dialog*roger.foreground:		#CD0000",
+"XScreenSaver.splash_dialog*roger.background:		#FFFFFF",
 "*demo_dialog*viewport.height:			200",
 "*Form.borderWidth:				0",
 "*Box.borderWidth:				0",
@@ -228,18 +273,23 @@
 "*resources_dialog*buttonbox.lock.label:		Require Password",
 "*resources_dialog*done.label:			OK",
 "*resources_dialog*cancel.label:			Cancel",
-"*passwd_dialog*label1.label:			XScreenSaver %s",
-"*passwd_dialog*label2.label:			This display is locked.",
-"*passwd_dialog*label3.label:		Please type %s's password to unlock it.",
-"*passwd_dialog*ok.label:			OK",
-"*passwd_dialog*cancel.label:			Cancel",
-"*passwd_dialog*passwd_form*label.label:		",
-"*passwd_dialog*Dialog.label:			",
-"*passwd_dialog*passwd_form*Text.width:		200",
-"*passwd_dialog*passwd_form.borderWidth:		0",
-"*passwd_dialog*roger.width:			150",
-"*passwd_dialog*roger.height:			150",
-"*passwd_dialog*roger.borderWidth:		1",
+"*splash_dialog*splashLabel1.label:		XScreenSaver %s",
+"*splash_dialog*splashLabel2.label:		Copyright  1991-1998 by",
+"*splash_dialog*splashLabel3.label:		Jamie Zawinski <jwz@jwz.org>",
+"*splash_dialog*splash_form*label.label:		",
+"*splash_dialog*Dialog.label:			",
+"*splash_dialog*splash_form.borderWidth:		0",
+"*splash_dialog*roger.width:			150",
+"*splash_dialog*roger.height:			150",
+"*splash_dialog*roger.borderWidth:		1",
+"*splash_dialog*demo.label:			Demo",
+"*splash_dialog*prefs.label:			Prefs",
+"*splash_dialog*help.label:			Help",
+"*splash_dialog.box.defaultDistance:		0",
+"*splash_dialog*splashLabel1.internalHeight:	10",
+"*splash_dialog*splashLabel1.internalWidth:	10",
+"*splash_dialog*splashLabel2.internalHeight:	0",
+"*splash_dialog*splashLabel3.internalHeight:	0",
 "XScreenSaver.pointerPollTime:		5",
 "XScreenSaver.initialDelay:		0",
 "XScreenSaver.windowCreationTimeout:	30",
