@@ -1,5 +1,5 @@
 /* dialogs-Xaw.c --- Athena widgets for demo, options, and password dialogs.
- * xscreensaver, Copyright (c) 1997 Jamie Zawinski <jwz@netscape.com>
+ * xscreensaver, Copyright (c) 1997, 1998 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -33,15 +33,6 @@
 
 #include "visual.h"	/* for visual_depth() */
 
-Widget passwd_dialog;
-Widget passwd_form;
-Widget roger_label;
-Widget passwd_label1;
-Widget passwd_label3;
-/*Widget passwd_text;*/
-/*Widget passwd_done;*/
-/*Widget passwd_cancel;*/
-
 Widget resources_dialog;
 Widget resources_form;
 Widget timeout_text;
@@ -73,67 +64,9 @@ Widget done;
 Widget restart;
 /*Widget spacer;*/
 
-
 static Widget buttonbox, textbox, okbox;
 
 
-void
-create_passwd_dialog(Widget parent, Visual *visual, Colormap colormap)
-{
-  Widget box, passwd_label2;
-  int depth = visual_depth(XtScreen(parent), visual);
-
-  passwd_dialog = 
-    XtVaCreatePopupShell("passwd_dialog", transientShellWidgetClass, parent,
-			 XtNtitle, NULL,
-			 XtNoverrideRedirect, TRUE,
-			 XtNvisual, visual,
-			 XtNcolormap, colormap,
-			 XtNdepth, depth,
-			 NULL);
-
-  box = XtVaCreateManagedWidget("box", formWidgetClass, passwd_dialog,
-			    XtNleft, XtChainLeft,
-			    XtNright, XtChainRight,
-			    XtNtop, XtChainTop,
-			    XtNbottom, XtChainBottom,
-			 XtNvisual, visual,
-			 XtNcolormap, colormap,
-			 XtNdepth, depth,
-			    NULL);
-
-  roger_label = XtVaCreateManagedWidget("roger", labelWidgetClass, box,
-					XtNlabel, "",
-					XtNleft, XtChainLeft,
-					XtNright, XtChainRight,
-					XtNtop, XtChainTop,
-					NULL);
-
-  passwd_label1 = XtVaCreateManagedWidget("label1", labelWidgetClass, box,
-					  XtNfromHoriz, roger_label,
-					  XtNright, XtChainRight,
-					  XtNtop, XtChainTop,
-					  NULL);
-  passwd_label2 = XtVaCreateManagedWidget("label2", labelWidgetClass, box,
-					  XtNfromHoriz, roger_label,
-					  XtNright, XtChainRight,
-					  XtNfromVert, passwd_label1,
-					  NULL);
-  passwd_label3 = XtVaCreateManagedWidget("label3", labelWidgetClass, box,
-					  XtNfromHoriz, roger_label,
-					  XtNright, XtChainRight,
-					  XtNfromVert, passwd_label2,
-					  NULL);
-  
-  passwd_form =
-    XtVaCreateManagedWidget("passwd_form", dialogWidgetClass, box,
-			    XtNfromHoriz, roger_label,
-			    XtNright, XtChainRight,
-			    XtNfromVert, passwd_label3,
-			    NULL);
-}
-
-
 void
 create_resources_dialog(Widget parent, Visual *visual, Colormap colormap)
 {
